#include "string_to.h"
#include "gmock/gmock.h"
#include "gtest/gtest.h"
#include <sstream>
#include <string>

using namespace std;

/* Test exact converstions from string to int (no unused characters). */
TEST(string_to_test, string_to_int_exact)
{
    int value = 0;

    EXPECT_FALSE(string_to<int>("", value));
    EXPECT_FALSE(string_to<int>("a", value));
    EXPECT_FALSE(string_to<int>("1b", value));
    EXPECT_FALSE(string_to<int>("a1b", value));

    EXPECT_TRUE(string_to<int>("-1", value));
    EXPECT_EQ(-1, value);

    EXPECT_TRUE(string_to<int>("0", value));
    EXPECT_EQ(0, value);

    EXPECT_TRUE(string_to<int>("1", value));
    EXPECT_EQ(1, value);
}

/* Test inexact converstions from string to int (trailing unused
 * characters are allowed). */
TEST(string_to_test, string_to_int_inexact)
{
    int value = 0;

    EXPECT_FALSE(string_to<int>("", value, /* exact */ false));
    EXPECT_FALSE(string_to<int>("a", value, /* exact */ false));
    EXPECT_FALSE(string_to<int>("a1", value, /* exact */ false));
    EXPECT_FALSE(string_to<int>("a1b", value, /* exact */ false));

    EXPECT_TRUE(string_to<int>("-1", value, /*exact */ false));
    EXPECT_EQ(-1, value);

    EXPECT_TRUE(string_to<int>("0", value, /*exact */ false));
    EXPECT_EQ(0, value);

    EXPECT_TRUE(string_to<int>("1", value, /*exact */ false));
    EXPECT_EQ(1, value);

    EXPECT_TRUE(string_to<int>("-1a", value, /* exact */ false));
    EXPECT_EQ(-1, value);

    EXPECT_TRUE(string_to<int>("0a", value, /* exact */ false));
    EXPECT_EQ(0, value);

    EXPECT_TRUE(string_to<int>("1a", value, /* exact */ false));
    EXPECT_EQ(1, value);
}

/* Test exact converstions from string to hex (no unused characters). */
TEST(string_to_test, string_to_hex_exact)
{
    unsigned int value = 0;

    EXPECT_FALSE(string_to<unsigned int>("", value));

    EXPECT_TRUE(string_to<unsigned int>(
        "0x1", value, /* exact */ true, std::hex));
    EXPECT_EQ((unsigned int)1, value);

    EXPECT_TRUE(string_to<unsigned int>(
        "0xa", value, /* exact */ true, std::hex));
    EXPECT_EQ((unsigned int)10, value);

    EXPECT_TRUE(string_to<unsigned int>(
        "0xA", value, /* exact */ true, std::hex));
    EXPECT_EQ((unsigned int)10, value);
}

/* Test string-to-string conversions. */
TEST(string_to_test, string_to_string)
{
    string value;

    EXPECT_FALSE(string_to<string>("", value));

    EXPECT_TRUE(string_to<string>("foo", value));
    EXPECT_STREQ("foo", value.c_str());

    EXPECT_TRUE(string_to<string>("123", value));
    EXPECT_STREQ("123", value.c_str());
}
